# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Luca Marchetti <ximilian@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gio, Gtk # pylint: disable=E0611

from dostuff_lib.helpers import get_builder

import gettext
from gettext import gettext as _
gettext.textdomain('dostuff')

import logging
logger = logging.getLogger('dostuff')

from dostuff_lib.AddDialog import AddDialog
from dostuff_lib.FilterDialog import FilterDialog
from dostuff_lib.Window import Window

import re

class AddDoStuffDialog(AddDialog):
    __gtype_name__ = "AddDoStuffDialog"
    task_line = ""
    id = None

    def finish_initializing(self, builder):
        """Set up the add dialog"""
        super(AddDoStuffDialog, self).finish_initializing(builder)
        self.cb_pri = builder.get_object("cb_priority")
        self.cb_pro = builder.get_object("cb_project")
        self.cb_con = builder.get_object("cb_context")
        self.tv_task = builder.get_object("tv_task")
        self.re_priority = re.compile("\(.\)")

    def reset(self):
        self.cb_pri.set_active(0)
        self.cb_pro.set_active(0)
        self.cb_con.set_active(0)
        self.tv_task.get_buffer().set_text("")
        self.id = None

    def update_choices(self, filter_catalog):

        self.cb_pri.get_model().clear()
        r = sorted(filter_catalog[FilterDialog._ID_PRIORITY], \
                key=lambda item: (int(item.partition(' ')[0]) \
                if item[0].isdigit() else float('inf'), item))
        self.cb_pri.append_text("Priority")
        for p in r:
            self.cb_pri.append_text(p)
        self.cb_pri.show()

        self.cb_pro.get_model().clear()
        r = sorted(filter_catalog[FilterDialog._ID_PROJECT], \
                key=lambda item: (int(item.partition(' ')[0]) \
                if item[0].isdigit() else float('inf'), item))
        self.cb_pro.append_text("Project")
        for p in r:
            self.cb_pro.append_text(p)
        self.cb_pro.show()

        self.cb_con.get_model().clear()
        r = sorted(filter_catalog[FilterDialog._ID_CONTEXT], \
                key=lambda item: (int(item.partition(' ')[0]) \
                if item[0].isdigit() else float('inf'), item))
        self.cb_con.append_text("Context")
        for c in r:
            self.cb_con.append_text(c)
        self.cb_con.show()

    def report_edit_signal(self, widget, data, pri, id):
        s = ""
        if not (pri == FilterDialog.PRI_NONE or pri == FilterDialog.PRI_COMPL):
            s += "(%s) " % pri
        s += data
        self.reset()
        self.id = id
        self.tv_task.get_buffer().set_text(s)
        self.show()

    def on_btn_reset_clicked(self, widget, data=None):
        self.reset()

    def on_btn_add_clicked(self, widget, data=None):
        self.task_line = self.tv_task.get_buffer().get_text(self.tv_task.get_buffer().get_start_iter(), self.tv_task.get_buffer().get_end_iter(), True)
        self.emit("add_signal", self.id)
        self.hide()

    def on_cb_priority_changed(self, widget, data=None):
        if self.cb_pri.get_active() > 0:
            s = self.cb_pri.get_active_text()
            text = self.tv_task.get_buffer().get_text(self.tv_task.get_buffer().get_start_iter(), self.tv_task.get_buffer().get_end_iter(), True)
            sl = text[:3]
            index_s = 0
            if self.re_priority.match(sl):
                index_s = 4
            self.tv_task.get_buffer().set_text("(" + s + ") " + text[index_s:])
        self.tv_task.grab_focus()


    def on_cb_project_changed(self, widget, data=None):
        if self.cb_pro.get_active() > 0:
            s = " +" + self.cb_pro.get_active_text()
            self.tv_task.get_buffer().insert_at_cursor(s)
        self.tv_task.grab_focus()

    def on_cb_context_changed(self, widget, data=None):
        if self.cb_con.get_active() > 0:
            s = " @" + self.cb_con.get_active_text()
            self.tv_task.get_buffer().insert_at_cursor(s)
        self.tv_task.grab_focus()
