# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Luca Marchetti <ximilian@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('dostuff')

from gi.repository import Gtk, Gio, GObject # pylint: disable=E0611
import logging
logger = logging.getLogger('dostuff')

from dostuff_lib import Window
from dostuff.AboutDoStuffDialog import AboutDoStuffDialog
from dostuff.PreferencesDoStuffDialog import PreferencesDoStuffDialog
from dostuff_lib.Task import Task
from dostuff.SortDoStuffWindow import SortDoStuffWindow
from dostuff.FilterDoStuffDialog import FilterDoStuffDialog
from dostuff.AddDoStuffDialog import AddDoStuffDialog

from datetime import date

import os

# See dostuff_lib.Window.py for more details about how this class works
class DoStuffWindow(Window):
    __gtype_name__ = "DoStuffWindow"
    I_ID = 0
    I_PRIORITY = 1
    I_DATA = 2
    I_PROJECT = 3
    I_CONTEXT = 4
    I_KEYWORD = 5
    I_COMPLETION_DATE = 6
    I_CREATION_DATE = 7

    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(DoStuffWindow, self).finish_initializing(builder)
        self.AboutDialog = AboutDoStuffDialog
        self.PreferencesDialog = PreferencesDoStuffDialog
        self.SortWindow = SortDoStuffWindow
        self.FilterDialog = FilterDoStuffDialog
        self.AddDialog = AddDoStuffDialog
        self.modify_task_window = self.builder.get_object("modify_task_window")
        
        # Code for other initialization actions should be added here.
        self.check_and_create_preferences_dialog()
        self.preferences_dialog.connect("pref_signal", self.report_pref_signal)
        self.check_and_create_sort_window() # create instance
        self.sort_window.connect("sort_signal", self.report_sort_signal)
        self.check_and_create_filter_dialog() # create instance
        self.filter_dialog.connect("filter_signal", self.report_filter_signal)
        self.check_and_create_add_dialog() # create instance
        self.add_dialog.connect("add_signal", self.report_add_signal)
        self.connect("edit_signal", self.add_dialog.report_edit_signal)

        self.treeview = self.builder.get_object("treeview_task")
        self.treeview.connect('button-press-event', self.modify_task_event)
        self.treemodel = self.treeview.get_model()

        # assign cellrenderer
        self.tvc_pri = self.builder.get_object("tvc_priority")
        self.tvc_pri.set_cell_data_func(self.builder.get_object("crt_pri"), self.format_priority_column)
        self.tvc_data = self.builder.get_object("tvc_data")
        self.tvc_data.set_cell_data_func(self.builder.get_object("crt_data"), self.format_data_column)

        self.treetaskfilter = self.treemodel.filter_new()
        self.treetaskfilter.set_visible_func(self.filter_task_cb, self.filter_dialog.filters)
        #self.treetasksort = Gtk.TreeModelSort(model=self.treetaskfilter)
        self.treeview.set_model(self.treetaskfilter)

        # start DoStuff
        self.init_tasks()


    def on_mnu_new_activate(self, widget, data=None):
        dialog = Gtk.FileChooserDialog("Please choose a file", self, \
                Gtk.FileChooserAction.SAVE, \
                (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, \
                Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
        dialog.set_current_name("todo.txt")
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            self.check_and_create_preferences_dialog()
            self.preferences_dialog.set_preferences("path", dialog.get_filename())
            #self.load_file(dialog.get_filename())
            self.preferences_dialog.save_preferences()
            self.reset()
        dialog.destroy()

    def on_mnu_open_activate(self, widget, data=None):
        dialog = Gtk.FileChooserDialog("Please choose a file", self, \
                Gtk.FileChooserAction.OPEN, \
                (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, \
                Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            self.check_and_create_preferences_dialog()
            self.preferences_dialog.set_preferences("path", dialog.get_filename())
            self.load_file(dialog.get_filename())
            self.preferences_dialog.save_preferences()
        dialog.destroy()

    def on_add_clicked(self, widget, data=None):
        self.check_and_create_add_dialog()
        self.add_dialog.show()

    def on_sort_clicked(self, widget, data=None):
        self.check_and_create_sort_window()
        self.sort_window.show()

    def on_filter_clicked(self, widget, data=None):
        self.check_and_create_filter_dialog()
        self.filter_dialog.show()

    def on_preferences_clicked(self, widget, data=None):
        self.check_and_create_preferences_dialog()
        self.preferences_dialog.show()

    def on_btn_done_clicked(self, widget, data=None):
        (model, iter) = self.treeview.get_selection().get_selected()
        it = model.convert_iter_to_child_iter(iter)
        self.treemodel.set(it, DoStuffWindow.I_PRIORITY, 'x')
        self.save_tasks()
        self.modify_task_window.hide()

    def on_btn_edit_clicked(self, widget, data=None):
        (model, iter) = self.treeview.get_selection().get_selected()
        it = model.convert_iter_to_child_iter(iter)
        self.emit("edit_signal", self.treemodel.get(it, DoStuffWindow.I_DATA)[0], self.treemodel.get(it, DoStuffWindow.I_PRIORITY)[0], it)
        self.modify_task_window.hide()

    def on_btn_delete_clicked(self, widget, data=None):
        (model, iter) = self.treeview.get_selection().get_selected()
        it = model.convert_iter_to_child_iter(iter)
        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.WARNING, \
            Gtk.ButtonsType.OK_CANCEL, "Preparing to delete a task")
        dialog.format_secondary_text("Would you really delete task '" + self.treemodel.get(it, DoStuffWindow.I_DATA)[0] + "'?")
        self.modify_task_window.hide()
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            self.treemodel.remove(it)
            self.save_tasks()
        #elif response == Gtk.ResponseType.CANCEL:
        #    print "WARN dialog closed by clicking CANCEL button"

        dialog.destroy()

    def on_modify_task_window_leave_notify_event(self, widget, data=None):
        (x, y) = widget.get_pointer()
        if x<0 or y<0 or y>=100 or x>=80: # see in glade
            self.modify_task_window.hide()

    def reset(self):
        self.treemodel.clear()
        self.filter_project = []
        self.filter_context = []
        self.filter_keyword = []
        self.filter_dialog.reset()
        self.add_dialog.reset()

    def init_tasks(self):
        # read task from file
        self.settings = Gio.Settings("net.launchpad.dostuff")
        file_path = self.settings.get_string("path")
        try:
            os.makedirs(os.path.dirname(file_path))
        except OSError:
            pass
        if os.path.exists(file_path):
            self.load_file(file_path)
            self.treemodel.set_sort_column_id(0, Gtk.SortType.ASCENDING)
        else:
            file = open(file_path, 'w')
            file.write('')
            file.close()
 

    def load_file (self, file_path):
        self.reset()
        # monitoring for changes
        self.task_file_path = file_path
        self.monitor = Gio.File.parse_name(file_path).monitor(Gio.FileMonitorFlags.NONE, None)
        self.monitor.connect ("changed", self.file_changed)

        for line in open(file_path):
            self.add_task(line)
        self.update_choices()

    def file_changed (self, monitor=None, changedfile=None, otherfile=None, event=None):
        if event == Gio.FileMonitorEvent.CHANGES_DONE_HINT:
            self.load_file(self.task_file_path)

    def update_choices(self):
        for p in self.filter_project:
            self.filter_dialog.add_filter_project(p)
        for c in self.filter_context:
            self.filter_dialog.add_filter_context(c)
        for k in self.filter_keyword:
            self.filter_dialog.add_filter_keyword(k)
        self.filter_dialog.update_choices()
        self.add_dialog.update_choices(self.filter_dialog.filter_catalog)

    def save_tasks(self):
        task_list = []
        for t in self.treemodel:
            s = ""
            if not t[DoStuffWindow.I_PRIORITY] == self.FilterDialog.PRI_NONE:
                if t[DoStuffWindow.I_PRIORITY] == self.FilterDialog.PRI_COMPL:
                    s = "x "
                else:
                    s = "(%s) " % t[DoStuffWindow.I_PRIORITY]
            if not t[DoStuffWindow.I_COMPLETION_DATE] == None:
                s += t[DoStuffWindow.I_COMPLETION_DATE].isoformat() + " "
            if not t[DoStuffWindow.I_CREATION_DATE] == None:
                s += t[DoStuffWindow.I_CREATION_DATE].isoformat() + " "
            s += t[DoStuffWindow.I_DATA]
            task_list.append([t[DoStuffWindow.I_ID], s])
        stask_list = sorted(task_list, key=lambda task: task[0])
        file_path = self.settings.get_string("path")
        ofile = open(file_path, "w")
        for l in stask_list:
            print>>ofile, l[1]

    def add_task(self, line, iter=None):
        t = Task(line.rstrip())
        if not t.data == "":
            creation_date = None
            if not t.creation_date == "":
                elem = t.creation_date.split("-")
                creation_date = date(int(elem[0]), int(elem[1]), int(elem[2]))
            completion_date = None
            if not t.completion_date == "":
                elem = t.completion_date.split("-")
                completion_date = date(int(elem[0]), int(elem[1]), int(elem[2]))
            if iter == None:
                self.treemodel.append([t.id, t.priority, t.data, t.project, t.context, t.keyword, completion_date, creation_date])
            else:
                self.treemodel.set(iter, DoStuffWindow.I_PRIORITY, t.priority)
                self.treemodel.set(iter, DoStuffWindow.I_DATA, t.data)
                self.treemodel.set(iter, DoStuffWindow.I_PROJECT, t.project)
                self.treemodel.set(iter, DoStuffWindow.I_CONTEXT, t.context)
                self.treemodel.set(iter, DoStuffWindow.I_KEYWORD, t.keyword)
                if not completion_date == None:
                    self.treemodel.set(iter, DoStuffWindow.I_COMPLETION_DATE, completion_date)
                if not creation_date == None:
                    self.treemodel.set(iter, DoStuffWindow.I_CREATION_DATE, creation_date)
            for p in t.project:
                self.filter_project.append(p)
            for c in t.context:
                self.filter_context.append(c)
            for k in t.keyword:
                self.filter_keyword.append(k)
            return True
        return False

    def modify_task_event(self, treeview, event):
        if event.button == 3: # single right click
            self.modify_task_window.show()
            self.modify_task_window.move(event.x-30, event.y+70)

    def report_pref_signal(self, data=None):
        self.init_tasks()

    def report_sort_signal(self, data=None):
        if self.sort_window.sort_id >=0 and self.sort_window.sort_id <3:
            self.treemodel.set_sort_column_id(self.sort_window.sort_id, Gtk.SortType.ASCENDING)
        self.sort_window.sort_id = -1

    def report_filter_signal(self, data=None):
        self.treetaskfilter.refilter()

    def report_add_signal(self, widget, id=None):
        if (self.add_task(self.add_dialog.task_line, id)):
            self.save_tasks()
        self.update_choices()
        self.treetaskfilter.refilter()

    def filter_task_cb(self, model, iter, data):
        return     (len(data[self.FilterDialog._ID_PRIORITY]) == 0 or \
                        self.treemodel.get_value(iter, 1) in data[self.FilterDialog._ID_PRIORITY]) \
               and (len(data[self.FilterDialog._ID_PROJECT]) == 0 or \
                    not data[self.FilterDialog._ID_PROJECT].isdisjoint(self.treemodel.get_value(iter, 3))) \
               and (len(data[self.FilterDialog._ID_CONTEXT]) == 0 or \
                    not data[self.FilterDialog._ID_CONTEXT].isdisjoint(self.treemodel.get_value(iter, 4))) \
               and (len(data[self.FilterDialog._ID_KEYWORD]) == 0 or \
                    not data[self.FilterDialog._ID_KEYWORD].isdisjoint(self.treemodel.get_value(iter, 5))) \
               and (not (self.settings.get_boolean("hide-completed") and \
                     model.get_value(iter, DoStuffWindow.I_PRIORITY) == 'x') ) \

    def format_priority_column(self, column, cell, model, iter, user_data):
        p = model.get_value(iter, DoStuffWindow.I_PRIORITY)
        text = "<span foreground='yellow'><b>"
        color = "white"
        if p == self.FilterDialog.PRI_COMPL or p == self.FilterDialog.PRI_NONE:
            p = ""
        if p == self.FilterDialog.PRI_A:
            color = "red"
        if p == self.FilterDialog.PRI_B:
            color = "orange"
        if p == self.FilterDialog.PRI_C:
            color = "yellow"
        text = "<span foreground='%s'><b>%s</b></span>" % (color, p)
        cell.set_property('markup', text)
        return
    
    def format_data_column(self, column, cell, model, iter, user_data):
        text = model.get_value(iter, DoStuffWindow.I_DATA)
        if model.get_value(iter, DoStuffWindow.I_PRIORITY) == 'x':
            text = "<span foreground='gray'><s><i>" + text + "</i></s></span>"
        else:
            t = ""
            for w in text.split():
                begin = w[:1]
                if begin == '+':
                    t += "<span foreground='gray'>" + w + "</span> "
                #elif begin == '#':
                    #t += "<span foreground='dark-red'>" + w + "</span> "
                elif begin == '@':
                    t += "<span foreground='gray'>" + w + "</span> "
                else:
                    t += w + " "
            text = t
        cell.set_property('markup', text)
        return 

