# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Luca Marchetti <ximilian@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gio, Gtk # pylint: disable=E0611

from dostuff_lib.helpers import get_builder

import gettext
from gettext import gettext as _
gettext.textdomain('dostuff')

import logging
logger = logging.getLogger('dostuff')

from dostuff_lib.FilterDialog import FilterDialog


class FilterDoStuffDialog(FilterDialog):
    __gtype_name__ = "FilterDoStuffDialog"

    def finish_initializing(self, builder):
        """Set up the filter dialog"""
        super(FilterDoStuffDialog, self).finish_initializing(builder)
        self.b_pri = builder.get_object("box_priority")
        self.b_pro = builder.get_object("box_project")
        self.b_con = builder.get_object("box_context")
        self.b_key = builder.get_object("box_keyword")


    def update_choices(self):
        [ self.b_pri.remove(w) for w in self.b_pri.get_children() ]
        r = sorted(self.filter_catalog[FilterDialog._ID_PRIORITY], \
                key=lambda item: (int(item.partition(' ')[0]) \
                if item[0].isdigit() else float('inf'), item))
        for p in r:
            if not (p == FilterDialog.PRI_NONE or p == FilterDialog.PRI_COMPL):
                button = Gtk.CheckButton(p)
                self.b_pri.pack_start(button, True, True, 0)
                button.show()
        self.b_pri.show()

        [ self.b_pro.remove(w) for w in self.b_pro.get_children() ]
        r = sorted(self.filter_catalog[FilterDialog._ID_PROJECT], \
                key=lambda item: (int(item.partition(' ')[0]) \
                if item[0].isdigit() else float('inf'), item))
        for p in r:
            button = Gtk.CheckButton(p)
            self.b_pro.pack_start(button, True, True, 0)
            button.show()
        self.b_pro.show()

        [ self.b_con.remove(w) for w in self.b_con.get_children() ]
        r = sorted(self.filter_catalog[FilterDialog._ID_CONTEXT], \
                key=lambda item: (int(item.partition(' ')[0]) \
                if item[0].isdigit() else float('inf'), item))
        for c in r:
            button = Gtk.CheckButton(c)
            self.b_con.pack_start(button, True, True, 0)
            button.show()
        self.b_con.show()

        [ self.b_key.remove(w) for w in self.b_key.get_children() ]
        r = sorted(self.filter_catalog[FilterDoStuffDialog._ID_KEYWORD], \
                key=lambda item: (int(item.partition(' ')[0]) \
                if item[0].isdigit() else float('inf'), item))
        for k in r:
            button = Gtk.CheckButton(k)
            self.b_key.pack_start(button, True, True, 0)
            button.show()
        self.b_key.show()

    def reset(self):
        self.resetfilters()
        [ f.clear() for f in self.filters ]
        [ w.set_active(False) for w in self.b_pri ]
        [ w.set_active(False) for w in self.b_pro ]
        [ w.set_active(False) for w in self.b_con ]
        [ w.set_active(False) for w in self.b_key ]

    def on_btn_reset_clicked(self, widget, data=None):
        self.reset()

    def on_btn_filter_clicked(self, widget, data=None):
        self.filters[FilterDoStuffDialog._ID_PRIORITY].clear()
        for w in self.b_pri:
            if w.get_active():
                self.filters[FilterDoStuffDialog._ID_PRIORITY].add(w.get_label())

        self.filters[FilterDoStuffDialog._ID_PROJECT].clear()
        for w in self.b_pro:
            if w.get_active():
                self.filters[FilterDoStuffDialog._ID_PROJECT].add(w.get_label())

        self.filters[FilterDoStuffDialog._ID_CONTEXT].clear()
        for w in self.b_con:
            if w.get_active():
                self.filters[FilterDoStuffDialog._ID_CONTEXT].add(w.get_label())

        self.filters[FilterDoStuffDialog._ID_KEYWORD].clear()
        for w in self.b_key:
            if w.get_active():
                self.filters[FilterDoStuffDialog._ID_KEYWORD].add(w.get_label())
        super(FilterDoStuffDialog, self).on_btn_filter_clicked(widget)
