# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Luca Marchetti <ximilian@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

# This is your preferences dialog.
#
# Define your preferences in
# data/glib-2.0/schemas/net.launchpad.dostuff.gschema.xml
# See http://developer.gnome.org/gio/stable/GSettings.html for more info.

from gi.repository import Gtk, Gio, GObject # pylint: disable=E0611

import xdg.BaseDirectory
from ConfigParser import ConfigParser
import os

import gettext
from gettext import gettext as _
gettext.textdomain('dostuff')

import logging
logger = logging.getLogger('dostuff')

from dostuff_lib.PreferencesDialog import PreferencesDialog

class PreferencesDoStuffDialog(PreferencesDialog):
    __gtype_name__ = "PreferencesDoStuffDialog"
    _conf_file = ""
    _config_parser = ConfigParser()

    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the preferences dialog"""
        super(PreferencesDoStuffDialog, self).finish_initializing(builder)

        conf_dir = os.path.join(xdg.BaseDirectory.xdg_config_home, "dostuff")
        self._conf_file = os.path.join(conf_dir, "dostuff.conf")
        try:
            os.makedirs(conf_dir)
        except OSError:
            pass

        # Bind each preference widget to gsettings
        self.settings = Gio.Settings("net.launchpad.dostuff")
        widget = self.builder.get_object('pref_path')
        self.settings.bind("path", widget, "text", Gio.SettingsBindFlags.DEFAULT)
        widget = self.builder.get_object('pref_hide_completed')
        self.settings.bind("hide-completed", widget, "active", Gio.SettingsBindFlags.DEFAULT)

        if not os.path.exists(self._conf_file):
            self._config_parser.add_section("general")
            self.set_preferences("path", os.path.join(conf_dir, "todo.txt"))
            self.set_preferences("hide-completed", True)
            self.save_preferences()

        self.load_preferences()

        # Code for other initialization actions should be added here.

    def on_btn_choose_file_clicked(self, widget, data=None):
        dialog = Gtk.FileChooserDialog("Please choose a file", self, \
                Gtk.FileChooserAction.OPEN, \
                (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, \
                Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            self.set_preferences("path", dialog.get_filename())
        dialog.destroy()

    def load_preferences(self):
        self._config_parser.read(self._conf_file)

        # Bind each preference widget to gsettings
        self.settings.set_string("path", self._config_parser.get("general", "path"))
        self.settings.set_boolean("hide-completed", self._config_parser.get("general", "hide-completed"))

    def save_preferences(self):
        cfile = open(self._conf_file, "w")
        value = self.settings.get_string("path")
        self._config_parser.set("general", "path", value)
        value = self.settings.get_boolean("hide-completed")
        self._config_parser.set("general", "hide-completed", value)
        self._config_parser.write(cfile)
        self.emit("pref_signal")

    def set_preferences(self, key, value):
        self._config_parser.set("general", key, value)
        if key == "hide-completed":
            self.settings.set_boolean(key, value)
        else:
            self.settings.set_string(key, value)

GObject.type_register(PreferencesDoStuffDialog)
GObject.signal_new("pref_signal", PreferencesDoStuffDialog, GObject.SIGNAL_RUN_FIRST, GObject.TYPE_NONE, ())
