# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Luca Marchetti <ximilian@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

"""this dialog adjusts values in gsettings
"""

from gi.repository import Gtk, GObject # pylint: disable=E0611
import logging
logger = logging.getLogger('dostuff_lib')

from . helpers import get_builder, show_uri, get_help_uri

class FilterDialog(Gtk.Dialog):
    __gtype_name__ = "FilterDialog"
    filters = []
    filter_catalog = []
    _ID_PRIORITY = 0
    _ID_PROJECT  = 1
    _ID_CONTEXT  = 2
    _ID_KEYWORD  = 3
    PRI_A = 'A'
    PRI_B = 'B'
    PRI_C = 'C'
    PRI_NONE = 'Z'
    PRI_COMPL = 'x'

    def __init__(self):
        self.filter_catalog = [ set(), set(), set(), set() ]
        self.filter_catalog[FilterDialog._ID_PRIORITY].add(FilterDialog.PRI_A)
        self.filter_catalog[FilterDialog._ID_PRIORITY].add(FilterDialog.PRI_B)
        self.filter_catalog[FilterDialog._ID_PRIORITY].add(FilterDialog.PRI_C)
        self.filter_catalog[FilterDialog._ID_PRIORITY].add(FilterDialog.PRI_COMPL)
        self.filters = [ set(), set(), set(), set() ]
        pass

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated FilterDialog object.
        """
        builder = get_builder('FilterDoStuffDialog')
        new_object = builder.get_object("filter_dostuff_dialog")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called while initializing this instance in __new__

        finish_initalizing should be called after parsing the ui definition
        and creating a FilterDialog object with it in order to
        finish initializing the start of the new FilterDoStuffDialog
        instance.
        
        Put your initialization code in here and leave __init__ undefined.
        """

        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self, True)

        # code for other initialization actions should be added here

    def update_choices(self):
        pass

    def resetfilters(self):
        [ s.clear() for s in self.filters ]

    def add_filter_project(self, data):
        self.add_filter(FilterDialog._ID_PROJECT, data)

    def add_filter_context(self, data):
        self.add_filter(FilterDialog._ID_CONTEXT, data)

    def add_filter_keyword(self, data):
        self.add_filter(FilterDialog._ID_KEYWORD, data)

    def add_filter(self, id, data):
        self.filter_catalog[id].add(data)

    def get_filter_priority(self):
        return self.get_filter(FilterDialog._ID_PRIORITY)

    def get_filter_project(self):
        return self.get_filter(FilterDialog._ID_PROJECT)

    def get_filter_context(self):
        return self.get_filter(FilterDialog._ID_CONTEXT)

    def get_filter_keyword(self):
        return self.get_filter(FilterDialog._ID_KEYWORD)

    def get_filter(self, data):
        return self.filters[data]

    def emit_filter_signal(self):
        print "emit_filter_signal"
        self.emit("filter_signal")
        self.hide()

    def on_btn_cancel_clicked(self, widget, data=None):
        self.hide()

    def on_btn_filter_clicked(self, widget, data=None):
        self.emit_filter_signal()

GObject.type_register(FilterDialog)
GObject.signal_new("filter_signal", FilterDialog, GObject.SIGNAL_RUN_FIRST, GObject.TYPE_NONE, ())
