# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Luca Marchetti <ximilian@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gio, Gtk, GObject # pylint: disable=E0611
import logging
logger = logging.getLogger('dostuff_lib')

from . helpers import get_builder, show_uri, get_help_uri

# This class is meant to be subclassed by DoStuffWindow.  It provides
# common functions and some boilerplate.
class SortWindow(Gtk.Window):
    __gtype_name__ = "SortWindow"
    sort_id = -1

    # To construct a new instance of this method, the following notable 
    # methods are called in this order:
    # __new__(cls)
    # __init__(self)
    # finish_initializing(self, builder)
    # __init__(self)
    #
    # For this reason, it's recommended you leave __init__ empty and put
    # your initialization code in finish_initializing
    def __init__(self):
        pass

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated BaseDoStuffWindow object.
        """
        builder = get_builder('SortDoStuffWindow')
        new_object = builder.get_object("sort_dostuff_window")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called while initializing this instance in __new__

        finish_initializing should be called after parsing the UI definition
        and creating a DoStuffWindow object with it in order to finish
        initializing the start of the new DoStuffWindow instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self, True)

    def emit_sort_signal(self, data):
        self.sort_id = data
        self.emit("sort_signal")


GObject.type_register(SortWindow)
GObject.signal_new("sort_signal", SortWindow, GObject.SIGNAL_RUN_FIRST, GObject.TYPE_NONE, ())
