# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Luca Marchetti <ximilian@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import re

class Task(object):
    """ A class for task object """

    id = -1
    priority = ""
    data = ""
    creation_date = ""
    completion_date = ""
    project = 0
    context = 0
    keyword = 0
    raw = ""

    status = "beg"

    OPEN_BRACKET  = "("
    CLOSE_BRACKET = ")"
    COMPLETED = "x"
    PROJECT   = "+"
    CONTEXT   = "@"
    KEYWORD   = "#"

    REGEX_DATE = re.compile('[0-9]{4}(-[0-9]{2}){2}')

    def __init__(self, line):
        Task.id += 1
        self.id = Task.id
        self.project = set()
        self.context = set()
        self.keyword = set()
        self.raw = line

        self.Parse()
        self.data = self.data.strip()
        if self.data == "":
            self.id = -1
            Task.id -=1

    def __repr__(self):
        return "Task(id=%r, priority=%r, data=%r, creation_date=%r, completion_date=%r, project=%r, context=%r, keyword=%r)" % \
                (self.id, self.priority, self.data, \
                 repr(self.creation_date), repr(self.completion_date), 
                 self.project, self.context, self.keyword)

    def __str__(self):
        return 'id: ' + str(self.id) + '\n' \
               'pri: ' + self.priority + '\n' \
               'date: ' + self.creation_date + " _ " + self.completion_date + '\n' \
               '+: ' + str(self.project) + '\n' \
               '@: ' + str(self.context) + '\n' \
               '#: ' + str(self.keyword) + '\n' \
               '[' + self.data + ']' + '\n'

    def Parse(self, begin=0):
        if not begin < self.raw.__len__():
            return

        while (self.raw[begin] == ' '):
            begin += 1

        if self.status == "beg":
            begin = self.ParsePriority(0)
            begin = self.ParseCompleted(begin)
        elif self.status == "sdate":
            [self.creation_date, begin] = self.ParseDate(begin)
            self.status = ""
        elif self.status == "edate":
            [self.completion_date, begin] = self.ParseDate(begin)
            self.status = "sdate"
        elif self.raw[begin] == Task.PROJECT:
            self.ParseToList(begin+1, self.project)
            begin = self.ParseDataWord(begin)
        elif self.raw[begin] == Task.CONTEXT:
            self.ParseToList(begin+1, self.context)
            begin = self.ParseDataWord(begin)
        elif self.raw[begin] == Task.KEYWORD:
            self.ParseToList(begin+1, self.keyword)
            begin = self.ParseDataWord(begin)
        else:
            begin = self.ParseDataWord(begin)

        self.Parse(begin)

    def ParsePriority(self, begin):
        self.status = "sdate"
        if self.raw[begin] == Task.OPEN_BRACKET and self.raw[begin+2] == Task.CLOSE_BRACKET:
            self.priority = self.raw[1]
            return begin+3
        self.priority = "Z"
        return begin

    def ParseCompleted(self, begin):
        if self.raw[begin] == Task.COMPLETED and self.raw[begin+1] == ' ':
            self.priority = Task.COMPLETED
            self.status = "edate"
            return begin+2
        return begin

    def ParseDate(self, begin):
        [word, end] = self.ParseWord(self.raw, begin)
        if not Task.REGEX_DATE.match(word):
            end  = begin
            word = ""
        return [word, end]

    def ParseToList(self, begin, working_list):
        [elem, end] = self.ParseWord(self.raw, begin)
        if not elem == "":
            working_list.add(elem)
        return end

    def ParseDataWord(self, begin):
        [word, end] = self.ParseWord(self.raw, begin)
        self.data += " " + word
        return end

    def ParseWord(self, text, begin):
        end = begin
        try:
            end = text.index(" ", begin)
        except (ValueError):
            end = text.__len__()
        word = text[begin:end]
        return [word, end]

